/**
* @file bootRomData.h
* @brief Common data and structures for booting from ROM
* @author Kosenkov
* @date 07.02.2018
*/

#ifndef __BOOTROMDATA_H__
#define __BOOTROMDATA_H__

#include "types.h"
#include "otpData.h"

/**
* Preinitialization data
*/
typedef struct
{
    uint32_t SN[SZ_SN];         /*< Current microcontroller serial number */
    uint32_t Crc;               /*< crc of serial number                  */
} PreInitData;
compiletime_assert((sizeof(PreInitData) >> 2) == (SZ_SN + 1));

/**
* Initialization data
*/
typedef struct
{
    uint32_t Gamma[SZ_GAMMA];   /*< Secret gamma   */
    uint32_t Crc;               /*< crc32 of gamma */
} InitData;
compiletime_assert((sizeof(InitData) >> 2) == (SZ_GAMMA + 1));

/**
* Work-state data
*/
#ifndef SMALL_GAMMA
typedef struct
{
    uint32_t CodeSz;            /*< size of code                                           */
    uint32_t Data[SZ_GAMMA - 6];/*< key, code, random words (CodeSz + CrcCrypted on gamma) */
    uint32_t HashCrypted[5];    /*< rmd160 of previous data. crypted on gamma              */
    uint32_t CrcOpened;         /*< crc32 of previous data. not crypted                    */
} WorkData;
compiletime_assert((sizeof(WorkData) >> 2) == (SZ_GAMMA + 1));
#else
typedef struct
{
    uint32_t CodeSz;            /*< size of code                                           */
    uint32_t Data[SZ_PROG - 6];/*< key, code, random words (CodeSz + CrcCrypted on gamma) */
    uint32_t HashCrypted[5];       /*< rmd160 of previous data. crypted on gamma                */
    uint32_t CrcOpened;         /*< crc32 of previous data. not crypted                    */
} WorkData;
#endif

#endif
