/**
* @file gate.h
* @brief gate header
* @author Kosenkov
* @date 01.02.2018
*/

#ifndef __GATE_H__
#define __GATE_H__

#include "types.h"

typedef enum
{
    GATE0 = 0,
    GATE1 = 1
} Gates;

/**
* Pushes 32-bits word to gate stack
*
* @param gate Gate number
* @param word To push
* @author Kosenkov
* @date 18.05.2018
*/
void gatePushWord(Gates gate, uint32_t word);

/**
* Reads 32-bits word from gate stack
*
* @param gate Gate number
* @param word To read to
* @author Kosenkov
* @date 18.05.2018
*/
void gatePopWord(Gates gate, uint32_t* word);

/**
* Pushes 32-bits words array to gate stack
*
* @param gate Gate number
* @param data Array to push
* @param sz Size of array
* @author Kosenkov
* @date 18.05.2018
*/
void gatePushData(Gates gate, uint32_t* data, Sz32 sz);

/**
* Reads 32-bits words array from gate stack
*
* @param gate Gate number
* @param data Array to read from stack
* @param sz Size of array
* @author Kosenkov
* @date 18.05.2018
*/
void gatePopData(Gates gate, uint32_t* data, Sz32 sz);

/**
* Set gate register to specified state
*
* @param gate Gate number
* @param regNum Register number
* @param word To store in register
* @author Kosenkov
* @date 18.05.2018
*/
void gateSetReg(Gates gate, uint8_t regNum, uint32_t word);

/**
* Get gate register state
*
* @param gate Gate number
* @param regNum Register number
* @param word To read from register
* @author Kosenkov
* @date 18.05.2018
*/
void gateGetReg(Gates gate, uint8_t regNum, uint32_t* word);

#endif
