/**
* @file uart.h
* @brief UART registers header
* @author Kosenkov
* @date 15.02.2018
*/

#ifndef __UART_H__
#define __UART_H__

#include "types.h"

/**
* UART module initialization
*
* @param clock Current clock on UART
* @param baudRate Desired baud rate
* @author Kosenkov
* @date 18.05.2018
*/
void uartInit(uint32_t clock, uint32_t baudRate);

/**
* UART module deinitialization
*
* @author Kosenkov
* @date 23.05.2018
*/
void uartDeinit(void);

/**
* Writes to UART. Blocking call
*
* @param data Data for writing to UART
* @param sz Size of data
* @author Kosenkov
* @date 18.05.2018
*/
void uartWrite(uint8_t* data, uint32_t sz);

/**
* Reads from UART. Blocking call
*
* @param data Data for reading from UART
* @param sz Size of data
* @author Kosenkov
* @date 18.05.2018
*/
void uartRead(uint8_t* data, uint32_t sz);

#endif
