; <<< Use Configuration Wizard in Context Menu >>>

; Amount of memory (in bytes) allocated for Stack
; Tailor this value to your application needs
; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

; 48KB
Stack_Size      EQU     0x0000C000

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

; 8KB
Heap_Size       EQU     0x00000000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit

                PRESERVE8
                THUMB

; <<< end of configuration section >>>

; CRC32
CRC32               EQU     0x83038583
    
; registers
SYS_PEN             EQU     0x40200008
RAM_SEED            EQU     0x4020001C
RNG_STAT_CTRL       EQU     0x4040C000
RNG_OUTPUT          EQU     0x4040C010

; masks
SYS_PEN_RANDOM      EQU     0x00400000
RNG_UNREAD_DATA     EQU     0x00000008
RNG_WORK_MODE_ONCE  EQU     0x00000100
    
; RAM init
RAM_START           EQU     0x20000000
RAM_END             EQU     0x20020000

; Vector Table Mapped to Address 0 at Reset
                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors
                EXPORT  __Vectors_End
                EXPORT  __Vectors_Size

__Vectors       DCD    __initial_sp                   ; Top of Stack
                DCD    Reset_Handler                  ; Reset Handler
                DCD    NMI_Handler                    ; NMI Handler
                DCD    HardFault_Handler              ; Hard Fault Handler
                DCD    CRC32                          ; CRC32
                DCD    0                              ; Reserved
                DCD    0                              ; Reserved
                DCD    0                              ; Reserved
                DCD    0                              ; Reserved
                DCD    0                              ; Reserved
                DCD    0                              ; Reserved
                DCD    SVC_Handler                    ; SVCall Handler
                DCD    0                              ; Reserved
                DCD    0                              ; Reserved
                DCD    PendSV_Handler                 ; PendSV Handler
                DCD    SysTick_Handler                ; SysTick Handler

                ; External Interrupts
                DCD    GATE0_FIFO_IRQHandler          ;  0: Gate Channel 0 FIFO Handler
                DCD    GATE0_REGS_IRQHandler          ;  1: Gate Channel 0 Registers Handler
                DCD    0                              ;  2: Reserved
                DCD    0                              ;  3: Reserved
                DCD    GATE1_FIFO_IRQHandler          ;  4: Gate Channel 1 FIFO Handler
                DCD    GATE1_REGS_IRQHandler          ;  5: Gate Channel 1 Registers Handler
                DCD    0                              ;  6: Reserved
                DCD    0                              ;  7: Reserved
                DCD    ALU_IRQHandler                 ;  8: ALU Handler
                DCD    0                              ;  9: Reserved
                DCD    0                              ; 10: Reserved
                DCD    0                              ; 11: Reserved
                DCD    LBLOCK_IRQHandler              ; 12: L Block Handler 
                DCD    0                              ; 13: Reserved
                DCD    0                              ; 14: Reserved
                DCD    0                              ; 15: Reserved
                DCD    GPIO_0_IRQHandler              ; 16: GPIO Handler pin 0
                DCD    GPIO_1_IRQHandler              ; 17: GPIO Handler pin 1
                DCD    GPIO_2_3_IRQHandler            ; 18: GPIO Handler pins 2 - 3
                DCD    GPIO_0_3_IRQHandler            ; 19: GPIO Handler pins 0 - 3
                DCD    GPIO_4_11_IRQHandler           ; 20: GPIO Handler pins 4 - 11
                DCD    GPIO_0_11_IRQHandler           ; 21: GPIO Handler pins 0 - 11
                DCD    0                              ; 22: Reserved
                DCD    0                              ; 23: Reserved
                DCD    UART_IRQHandler                ; 24: UART Handler 
                DCD    0                              ; 25: Reserved
                DCD    0                              ; 26: Reserved
                DCD    0                              ; 27: Reserved
                DCD    RND_IRQHandler                 ; 28: Random Sensor Handler 
                DCD    0                              ; 29: Reserved
                DCD    0                              ; 30: Reserved
                DCD    0                              ; 31: Reserved


__Vectors_End

__Vectors_Size  EQU  __Vectors_End - __Vectors

                AREA    |.text|, CODE, READONLY

; Empty system init
SystemInit      PROC
                EXPORT  SystemInit                      [WEAK]
                BX      LR
                ENDP

; Reset handler routine
Reset_Handler   PROC
                EXPORT  Reset_Handler                   [WEAK]
        IMPORT  __main         
        IMPORT  main         
			
;                ; switch on rnd
;                LDR     R0, =SYS_PEN
;                LDR     R1, [R0]
;                LDR     R2, =SYS_PEN_RANDOM
;                ORRS    R1, R2
;                STR     R1, [R0]
;                
;                ; wait for initialization
;                EORS    R2, R2
;                LDR     R0, =RNG_STAT_CTRL
;RngInitLoop     LDR     R1, [R0]
;                CMP     R1, R2
;                BEQ     RngInitLoop
;                
;                ; get rnd
;                LDR     R2, =RNG_WORK_MODE_ONCE
;                ORRS    R1, R2
;                STR     R1, [R0]
;                
;                LDR     R2, =RNG_UNREAD_DATA
;RngGenLoop      LDR     R1, [R0]
;                ANDS    R1, R2
;                CMP     R1, R2
;                BNE     RngGenLoop

;                LDR     R1, =RNG_OUTPUT
;                LDR     R1, [R1]
;                LDR     R0, =RAM_SEED
;                STR     R1, [R0]
;                
;                ; RAM init
;                LDR     R0, =RAM_START
;                LDR     R1, =RAM_END
;                MOVS    R2, #0
;				
;RamInit
;                STR     R2, [R0]
;                ADDS    R0, #4
;                CMP     R0, R1
;                BNE     RamInit

;                LDR     R0, =SystemInit
;                BLX     R0
                LDR     R0, =main
                BX      R0
                ENDP


; Dummy Exception Handlers (infinite loops which can be modified)

NMI_Handler     PROC
                EXPORT  NMI_Handler                      [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler                [WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler                      [WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler                   [WEAK]
                B       .
                ENDP
SysTick_Handler PROC
                EXPORT  SysTick_Handler                  [WEAK]
                B       .
                ENDP

Default_Handler PROC

                EXPORT  GATE0_FIFO_IRQHandler            [WEAK]
                EXPORT  GATE0_REGS_IRQHandler            [WEAK]
                EXPORT  GATE1_FIFO_IRQHandler            [WEAK]
                EXPORT  GATE1_REGS_IRQHandler            [WEAK]
                EXPORT  ALU_IRQHandler                   [WEAK]
                EXPORT  LBLOCK_IRQHandler                [WEAK]
                EXPORT  GPIO_0_IRQHandler                [WEAK]
                EXPORT  GPIO_1_IRQHandler                [WEAK]
                EXPORT  GPIO_2_3_IRQHandler              [WEAK]
                EXPORT  GPIO_0_3_IRQHandler              [WEAK]
                EXPORT  GPIO_4_11_IRQHandler             [WEAK]
                EXPORT  GPIO_0_11_IRQHandler             [WEAK]
                EXPORT  UART_IRQHandler                  [WEAK]
                EXPORT  RND_IRQHandler                   [WEAK]


GATE0_FIFO_IRQHandler
GATE0_REGS_IRQHandler
GATE1_FIFO_IRQHandler
GATE1_REGS_IRQHandler
ALU_IRQHandler
LBLOCK_IRQHandler
GPIO_0_IRQHandler    
GPIO_1_IRQHandler    
GPIO_2_3_IRQHandler  
GPIO_0_3_IRQHandler  
GPIO_4_11_IRQHandler 
GPIO_0_11_IRQHandler 
UART_IRQHandler
RND_IRQHandler

                B       .
                ENDP
                ALIGN

;*******************************************************************************
; User Stack and Heap initialization
;*******************************************************************************
                 IF      :DEF:__MICROLIB

                 EXPORT  __initial_sp
                 EXPORT  __heap_base
                 EXPORT  __heap_limit

                 ELSE

                 IMPORT  __use_two_region_memory
                 EXPORT  __user_initial_stackheap

__user_initial_stackheap

                 LDR     R0, = Heap_Mem
                 LDR     R1, = (Stack_Mem + Stack_Size)
                 LDR     R2, = (Heap_Mem + Heap_Size)
                 LDR     R3, = Stack_Mem
                 BX      LR

                 ALIGN

                 ENDIF

                 END


;*******************************************************************************
